from django.urls import include, path
from utilities.urls import get_model_urls

import netbox_rpki
# from netbox_rpki.models import (
#    Organization, Certificate, Roa, RoaPrefix
# )
from netbox_rpki import views

app_name = 'netbox_rpki'

urlpatterns = [
    # certificate
    path('certificate/', netbox_rpki.views.CertificateListView.as_view(), name='certificate_list'),
    path('certificate/add/', views.CertificateEditView.as_view(), name='certificate_add'),
    path('certificate/<int:pk>/', views.CertificateView.as_view(), name='certificate'),
    path('certificate/<int:pk>/edit/', views.CertificateEditView.as_view(), name='certificate_edit'),
    path('certificate/<int:pk>/delete/', views.CertificateDeleteView.as_view(), name='certificate_delete'),
    path('certificate/<int:pk>/', include(get_model_urls('netbox_rpki', 'certificate'))),
    # organization
    path('orgs/', netbox_rpki.views.OrganizationListView.as_view(), name='organization_list'),
    path('orgs/add/', netbox_rpki.views.OrganizationEditView.as_view(), name='organization_add'),
    path('orgs/<int:pk>/', netbox_rpki.views.OrganizationView.as_view(), name='organization'),
    path('orgs/<int:pk>/edit/', netbox_rpki.views.OrganizationEditView.as_view(), name='organization_edit'),
    path('orgs/<int:pk>/delete/', netbox_rpki.views.OrganizationDeleteView.as_view(), name='organization_delete'),
    path('orgs/<int:pk>/', include(get_model_urls('netbox_rpki', 'organization'))),
    # roa
    path('roa/', views.RoaListView.as_view(), name='roa_list'),
    path('roa/add/', views.RoaEditView.as_view(), name='roa_add'),
    path('roa/<int:pk>/', views.RoaView.as_view(), name='roa'),
    path('roa/<int:pk>/edit/', views.RoaEditView.as_view(), name='roa_edit'),
    path('roa/<int:pk>/delete/', views.RoaDeleteView.as_view(), name='roa_delete'),
    path('roa/<int:pk>/', include(get_model_urls('netbox_rpki', 'roa'))),
    # roaprefix
    path('roaprefices/', views.RoaPrefixListView.as_view(), name='roaprefix_list'),
    path('roaprefices/add/', views.RoaPrefixEditView.as_view(), name='roaprefix_add'),
    path('roaprefices/<int:pk>/', views.RoaPrefixView.as_view(), name='roaprefix'),
    path('roaprefices/<int:pk>/edit/', views.RoaPrefixEditView.as_view(), name='roaprefix_edit'),
    path('roaprefices/<int:pk>/delete/', views.RoaPrefixDeleteView.as_view(), name='roaprefix_delete'),
    path('roaprefices/<int:pk>/', include(get_model_urls('netbox_rpki', 'roaprefix'))),
]



from django.db import models
from django.urls import reverse
from netbox.models import NetBoxModel
from ipam.models.asns import ASN
from ipam.models.ip import Prefix
from ipam.models import RIR


class Organization(NetBoxModel):
    org_id = models.CharField(max_length=200, editable=True)
    name = models.CharField(max_length=200, editable=True)
    ext_url = models.CharField(max_length=200, editable=True, blank=True)
    parent_rir = models.ForeignKey(
        to=RIR,
        on_delete=models.PROTECT,
        related_name='rpki_certs',
        null=True,
        blank=True
    )

    class Meta:
        ordering = ("name",)

    def __str__(self):
        return f'{self.name}, {self.org_id}'

    def get_absolute_url(self):
        return reverse("plugins:netbox_rpki:organization", args=[self.pk])


class Certificate(NetBoxModel):
    name = models.CharField(max_length=200, editable=True)
    issuer = models.CharField(max_length=200, editable=True, blank=True)
    subject = models.CharField(max_length=200, editable=True, blank=True)
    serial = models.CharField(max_length=200, editable=True, blank=True)
    valid_from = models.DateField(editable=True, blank=True, null=True)
    valid_to = models.DateField(editable=True, blank=True, null=True)
    auto_renews = models.BooleanField(editable=True)
    public_key = models.CharField(editable=True, blank=True)
    private_key = models.CharField(editable=True, blank=True)
    publication_url = models.CharField(editable=True, blank=True)
    ca_repository = models.CharField(editable=True, blank=True)
    self_hosted = models.BooleanField(editable=True)
    rpki_org = models.ForeignKey(
        to=Organization,
        on_delete=models.CASCADE,
        related_name='certificates'
    )

    class Meta:
        ordering = ("name",)

    def __str__(self):
        return f'{self.name}, {self.issuer}'

    def get_absolute_url(self):
        return reverse("plugins:netbox_rpki:certificate", args=[self.pk])


class Roa(NetBoxModel):
    name = models.CharField(max_length=200, editable=True)
    origin_as = models.ForeignKey(
        to=ASN,
        on_delete=models.CASCADE,
        related_name='roas',
        blank=True,
        null=True
    )
    valid_from = models.DateField(editable=True, blank=True, null=True)
    valid_to = models.DateField(editable=True, blank=True, null=True)
    auto_renews = models.BooleanField(editable=True)
    signed_by = models.ForeignKey(
        to=Certificate,
        on_delete=models.PROTECT,
        related_name='roas'
    )

    class Meta:
        ordering = ("name",)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse("plugins:netbox_rpki:roa", args=[self.pk])


class RoaPrefix(NetBoxModel):
    prefix = models.ForeignKey(
        to=Prefix,
        on_delete=models.PROTECT,
        related_name='RoaPrefices'
    )
    max_length = models.IntegerField(editable=True)
    roa_name = models.ForeignKey(
        to=Roa,
        on_delete=models.PROTECT,
        related_name='prefices'
    )

    class Meta:
        ordering = ("prefix",)

    def __str__(self):
        return str(self.prefix)

    def get_absolute_url(self):
        return reverse("plugins:netbox_rpki:roaprefix", args=[self.pk])




import django_tables2 as tables
from netbox.tables import NetBoxTable
# , ChoiceFieldColumn
import netbox_rpki
# from netbox_rpki.models import Certificate, Organization, Roa, RoaPrefix


class CertificateTable(NetBoxTable):
    name = tables.Column(linkify=True)

    class Meta(NetBoxTable.Meta):
        model = netbox_rpki.models.Certificate
        fields = ("pk", "id", "name", "issuer", "subject", "serial", "valid_from", "valid_to", "auto_renews", "publicKey", "private_key", "publication_url", "ca_repository", "self_hosted", "rpki_org")
        default_columns = ("name", "issuer", "subject", "serial", "valid_from", "valid_to", "auto_renews", "publicKey", "private_key", "publication_url;", "ca_repository", "self_hosted", "rpki_org")


class OrganizationTable(NetBoxTable):
    name = tables.Column(linkify=True)

    class Meta(NetBoxTable.Meta):
        model = netbox_rpki.models.Organization
        fields = ("pk", "id", "org_id", "name", "parent_rir", "ext_url")
        default_columns = ("org_id", "name","parent_rir", "ext_url")


class RoaTable(NetBoxTable):
    name = tables.Column(linkify=True)

    class Meta(NetBoxTable.Meta):
        model = netbox_rpki.models.Roa
        fields = ("pk", "id", 'name', "origin_as", "valid_from", "valid_to", "auto_renews", "signed_by")
        default_columns = ("name", "origin_as", "valid_from", "valid_to", "auto_renews", "signed_by")


class RoaPrefixTable(NetBoxTable):
    pk = tables.Column(linkify=True)

    class Meta(NetBoxTable.Meta):
        model = netbox_rpki.models.RoaPrefix
        fields = ("pk", "id", "prefix", "max_length", "roa_name")
        default_columns = ("prefix", "max_length", "roa_name")



import netbox_rpki
from netbox.views import generic
from netbox_rpki.models import Certificate, Organization, Roa, RoaPrefix
from netbox_rpki.forms import CertificateForm, OrganizationForm, RoaForm, RoaPrefixForm
from netbox_rpki.tables import CertificateTable, OrganizationTable, RoaTable, RoaPrefixTable


class CertificateView(generic.ObjectView):
    queryset = netbox_rpki.models.Certificate.objects.all()

    def get_extra_context(self, request, instance):
        roa_table = netbox_rpki.tables.RoaTable(instance.roas.all())
        roa_table.configure(request)

        return {
            'signed_roas_table': roa_table,
        }


class CertificateListView(generic.ObjectListView):
    queryset = Certificate.objects.all()
    table = CertificateTable


class CertificateEditView(generic.ObjectEditView):
    queryset = Certificate.objects.all()
    form = CertificateForm


class CertificateDeleteView(generic.ObjectDeleteView):
    queryset = Certificate.objects.all()


class OrganizationView(generic.ObjectView):
    queryset = Organization.objects.all()


    def get_extra_context(self, request, instance):
        mycerts_table = netbox_rpki.tables.CertificateTable(instance.certificates.all())
        mycerts_table.configure(request)

        return {
            'certificates_table': mycerts_table,
        }


class OrganizationListView(generic.ObjectListView):
    queryset = Organization.objects.all()
    table = OrganizationTable


class OrganizationEditView(generic.ObjectEditView):
    queryset = Organization.objects.all()
    form = OrganizationForm


class OrganizationDeleteView(generic.ObjectDeleteView):
    queryset = Organization.objects.all()


class RoaPrefixView(generic.ObjectView):
    queryset = RoaPrefix.objects.all()


class RoaPrefixListView(generic.ObjectListView):
    queryset = RoaPrefix.objects.all()
    table = RoaPrefixTable


class RoaPrefixEditView(generic.ObjectEditView):
    queryset = RoaPrefix.objects.all()
    form = RoaPrefixForm


class RoaPrefixDeleteView(generic.ObjectDeleteView):
    queryset = RoaPrefix.objects.all()


class RoaView(generic.ObjectView):
    queryset = Roa.objects.all()


    def get_extra_context(self, request, instance):
        roaprefix_table = netbox_rpki.tables.RoaPrefixTable(instance.prefices.all())
        roaprefix_table.configure(request)

        return {
            'roaprefices_table': roaprefix_table
        }


class RoaListView(generic.ObjectListView):
    queryset = Roa.objects.all()
    table = RoaTable


class RoaEditView(generic.ObjectEditView):
    queryset = Roa.objects.all()
    form = RoaForm


class RoaDeleteView(generic.ObjectDeleteView):
    queryset = Roa.objects.all()



from rest_framework.routers import APIRootView
from netbox.api.viewsets import NetBoxModelViewSet
from netbox_rpki.api.serializers import OrganizationSerializer, CertificateSerializer, RoaSerializer, RoaPrefixSerializer
from netbox_rpki import filtersets, models


class RootView(APIRootView):
    def get_view_name(self):
        return 'rpki'


class OrganizationViewSet(NetBoxModelViewSet):
    queryset = models.Organization.objects.all()
    serializer_class = OrganizationSerializer
    filterset_class = filtersets.OrganizationFilterSet


class CertificateViewSet(NetBoxModelViewSet):
    queryset = models.Certificate.objects.all()
    serializer_class = CertificateSerializer
    filterset_class = filtersets.CertificateFilterSet


class RoaViewSet(NetBoxModelViewSet):
    queryset: models.Roa.objects.all()
    serializer_class = RoaSerializer
    filterset_class = filtersets.RoaFilterSet


class RoaPrefixViewSet(NetBoxModelViewSet):
    queryset: models.RoaPrefix.objects.all()
    serializer_class = RoaPrefixSerializer
    filterset_class = filtersets.RoaPrefixFilterSet



__author__ = """H.L. Mencken Davidson"""
__email__ = "netbox_rpki@toomanydavidsons.com"

from netbox.plugins import PluginConfig
from netbox_rpki._version import __version__

# import api

class rpki_config(PluginConfig):
    name = 'netbox_rpki'
    verbose_name = 'RPKI functionality for Netbox'
    description = 'Add RPKI data elements to Netbox.'
    version = __version__
    author = 'Mencken Davidson'
    author_email = 'mencken@gmail.com'
    base_url = 'netbox_rpki'
    default_settings = {
        'top_level_menu': True
        }

config = rpki_config



# from rest_framework.relations import PrimaryKeyRelatedField
# from netbox.api.fields import ChoiceField, SerializedPKRelatedField
from netbox.api.serializers import NetBoxModelSerializer
from rest_framework.serializers import HyperlinkedIdentityField  # , ValidationError

# from ipam.api.serializers import IPAddressSerializer, ASNSerializer, PrefixSerializer
# from tenancy.api.serializers import TenantSerializer
# from dcim.api.serializers import SiteSerializer, DeviceSerializer
import netbox_rpki

# from netbox_rpki.models import Certificate, Organization, Roa, RoaPrefix


class CertificateSerializer(NetBoxModelSerializer):
    url = HyperlinkedIdentityField(view_name="plugins-api:netbox_rpki:certificate-detail")

    class Meta:
        model = netbox_rpki.models.Certificate
        fields = ("id", "name", "issuer", "subject", "serial", "valid_from", "valid_to", "auto_renews", "public_key", "private_key", "publication_url", "ca_repository", "self_hosted", "rpki_org")
        brief_fields = ("name", "issuer", "subject", "serial", "rpki_org")


class OrganizationSerializer(NetBoxModelSerializer):
    url = HyperlinkedIdentityField(view_name="plugins-api:netbox_rpki_:organization-detail")

    class Meta:
        model = netbox_rpki.models.Organization
        fields = ["id", "org_id", "name", "ext_url", "parent_rir"]
        brief_fields = ("org_id", "name", "parent_rir")


class RoaSerializer(NetBoxModelSerializer):
    url = HyperlinkedIdentityField(view_name="plugins-api:netbox_rpki_:roa-detail")

    class Meta:
        model = netbox_rpki.models.Roa
        fields = ["id", "name", "origin_as", "valid_from", "valid_to", "auto_renews", "signed_by"]
        brief_fields = ("name", "origin_as")


class RoaPrefixSerializer(NetBoxModelSerializer):
    url = HyperlinkedIdentityField(view_name="plugins-api:netbox_rpki_:roaprefix-detail")

    class Meta:
        model = netbox_rpki.models.RoaPrefix
        fields = ["id", "prefix", "max_length", "roa_name"]
        brief_fields = ("id", "prefix", "max_length", "roa_name")



"""Django API url router definitions for the netbox_ptov plugin"""

from netbox.api.routers import NetBoxRouter
from netbox_rpki.api.views import CertificateViewSet, OrganizationViewSet, RoaViewSet, RoaPrefixViewSet, RootView

app_name = 'netbox_rpki'

router = NetBoxRouter()
router.APIRootView = RootView
router.register('certificate', CertificateViewSet, basename='certificate')
router.register('organization', OrganizationViewSet, basename='organization')
router.register('roa', RoaViewSet, basename='roa')
router.register('roaprefix', RoaPrefixViewSet, basename='roaprefix')

urlpatterns = router.urls



from http import HTTPStatus

from django.test import TestCase

from netbox_rpki.forms import gns3srvForm


class TestGns3srvFormCase(TestCase):
    def test_gns3srv_valid(self):
        form = gns3srvForm(
            data={
                'value': 'myserver.notyourserver',
                'status': 'active'
            }
        )
        self.assertEqual(
            form.errors.get('value'), None
        )



from netbox.filtersets import NetBoxModelFilterSet

import netbox_rpki

# from netbox_rpki.models import Certificate, Organization, Roa, RoaPrefix


class CertificateFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = netbox_rpki.models.Certificate
        fields = ['name', 'issuer', 'subject', 'serial', 'valid_from', 'valid_to', 'public_key', 'private_key', 'publication_url', 'ca_repository', 'rpki_org', 'self_hosted']

    def search(self, queryset, name, value):
        return queryset.filter(description__icontains=value)


class OrganizationFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = netbox_rpki.models.Organization
        fields = ['org_id', 'name', 'parent_rir', 'ext_url']

    def search(self, queryset, name, value):
        return queryset.filter(description__icontains=value)


class RoaFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = netbox_rpki.models.Roa
        fields = ['name', 'origin_as', 'valid_from', 'valid_to', 'signed_by']

    def search(self, queryset, name, value):
        return queryset.filter(description__icontains=value)


class RoaPrefixFilterSet(NetBoxModelFilterSet):
    class Meta:
        model = netbox_rpki.models.RoaPrefix
        fields = ['prefix', 'max_length', 'roa_name']

    def search(self, queryset, name, value):
        return queryset.filter(description__icontains=value)



from django.conf import settings

from netbox.plugins import PluginMenuButton, PluginMenuItem, PluginMenu


_menu_items = (
    PluginMenuItem(
        link='plugins:netbox_rpki:organization_list',
        link_text='Organizations',
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_rpki:organization_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
            ),
        ),
    ),

    PluginMenuItem(
        link='plugins:netbox_rpki:certificate_list',
        link_text='Customer Certificates',
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_rpki:certificate_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
            ),
        ),
    ),

    PluginMenuItem(
        link='plugins:netbox_rpki:roa_list',
        link_text='ROAs',
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_rpki:roa_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_rpki:roaprefix_list',
        link_text='ROA Prefices',
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_rpki:roaprefix_add',
                title='Add',
                icon_class='mdi mdi-plus-thick',
            ),
        ),
    )

)
plugin_settings = settings.PLUGINS_CONFIG.get('netbox_rpki', {})

if plugin_settings.get('top_level_menu'):
    menu = PluginMenu(
        label="RPKI",
        groups=(("RPKI", _menu_items),),
        icon_class="mdi mdi-bootstrap",
    )
else:
    menu_items = _menu_items



# Generated by Django 5.0.9 on 2024-11-04 03:31

import django.db.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('extras', '0121_customfield_related_object_filter'),
        ('ipam', '0070_vlangroup_vlan_id_ranges'),
    ]

    operations = [
        migrations.CreateModel(
            name='Organization',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('org_id', models.CharField(max_length=200)),
                ('name', models.CharField(max_length=200)),
                ('ext_url', models.CharField(blank=True, max_length=200)),
                ('parent_rir', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='rpki_certs', to='ipam.rir')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'ordering': ('name',),
            },
        ),
        migrations.CreateModel(
            name='Certificate',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=200)),
                ('issuer', models.CharField(blank=True, max_length=200)),
                ('subject', models.CharField(blank=True, max_length=200)),
                ('serial', models.CharField(blank=True, max_length=200)),
                ('valid_from', models.DateField(blank=True, null=True)),
                ('valid_to', models.DateField(blank=True, null=True)),
                ('auto_renews', models.BooleanField()),
                ('public_key', models.CharField(blank=True)),
                ('private_key', models.CharField(blank=True)),
                ('publication_url', models.CharField(blank=True)),
                ('ca_repository', models.CharField(blank=True)),
                ('self_hosted', models.BooleanField()),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('rpki_org', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='certificates', to='netbox_rpki.organization')),
            ],
            options={
                'ordering': ('name',),
            },
        ),
        migrations.CreateModel(
            name='Roa',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('name', models.CharField(max_length=200)),
                ('valid_from', models.DateField(blank=True, null=True)),
                ('valid_to', models.DateField(blank=True, null=True)),
                ('auto_renews', models.BooleanField()),
                ('origin_as', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='roas', to='ipam.asn')),
                ('signed_by', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='roas', to='netbox_rpki.certificate')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'ordering': ('name',),
            },
        ),
        migrations.CreateModel(
            name='RoaPrefix',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('max_length', models.IntegerField()),
                ('prefix', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='RoaPrefices', to='ipam.prefix')),
                ('roa_name', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='prefices', to='netbox_rpki.roa')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'ordering': ('prefix',),
            },
        ),
    ]



# import netbox_rpki
# from django import forms
# from ipam.models import Prefix
from netbox.forms import NetBoxModelForm

# from utilities.forms.fields import CommentField, DynamicModelChoiceField
# from dcim.models import devices
# from netbox_rpki.models import Certificate, Organization, Roa, RoaPrefix
from netbox_rpki.models import Certificate, Organization, Roa, RoaPrefix


class CertificateForm(NetBoxModelForm):

    class Meta:
        model = Certificate
        fields = ['name', 'issuer', 'subject', 'serial', 'valid_from', 'valid_to', "auto_renews", 'public_key', 'private_key', 'publication_url', 'ca_repository', 'rpki_org', 'self_hosted']


class OrganizationForm(NetBoxModelForm):

    class Meta:
        model = Organization
        fields = ['org_id', 'name', 'parent_rir', 'ext_url']


class RoaForm(NetBoxModelForm):

    class Meta:
        model = Roa
        fields: list[str] = ['name', 'origin_as', 'valid_from', 'valid_to', "auto_renews", 'signed_by']


class RoaPrefixForm(NetBoxModelForm):

    class Meta:
        model = RoaPrefix
        fields = ['prefix', 'max_length', 'roa_name']
